import dataclasses

from plmdp.models.type_agnostic import BaseProfile
from plmdp.primitives import OptionalDateOrDateTime


@dataclasses.dataclass
class DateOrDateTimeProfile(BaseProfile):
    min: OptionalDateOrDateTime
    max: OptionalDateOrDateTime



import polars
from typing import Callable

from plmdp.exceptions import EmptyDataFrameException, NoColumnsToProfile
from plmdp.models.output import ProfilerOutput, ColumnData
from plmdp.profile_factory import MetricsHandlerFactory
from plmdp.models import BaseProfile

import logging

logger = logging.getLogger(__name__)


class Profiler:
    def __init__(
        self,
        profile_factory: MetricsHandlerFactory | None = None,
    ):
        self._profile_factory = (
            profile_factory if profile_factory else MetricsHandlerFactory()
        )

    def _get_handler(
        self,
        data_type: polars.datatypes.DataType,
    ) -> Callable[[polars.DataFrame, str], BaseProfile]:
        return self._profile_factory.create(data_type)

    @staticmethod
    def _validate_row_count_not_zero(
        dataframe: polars.DataFrame,
    ) -> None:
        if dataframe.shape[0] == 0:
            msg = "Passed dataframe is empty."
            logger.error(msg)
            raise EmptyDataFrameException(msg)

    @staticmethod
    def _validate_columns_to_profile(
        dataframe: polars.DataFrame,
        columns_to_ignore: list[str],
    ) -> None:
        dataframe_columns = set(dataframe.columns)
        ignored_columns = set(columns_to_ignore)

        if dataframe_columns.issubset(ignored_columns):
            msg = "All columns are ignored, no columns left to profile."
            logger.error(msg)
            raise NoColumnsToProfile(msg)

        extra_ignored_columns = ignored_columns - dataframe_columns
        if extra_ignored_columns:
            logger.warning(
                f"Warning: The following columns in 'columns_to_ignore' are not in the DataFrame: {extra_ignored_columns}"
            )

    def _gather_columns_metrics(
        self,
        dataframe: polars.DataFrame,
        columns_to_ignore: list[str],
    ) -> list[ColumnData]:
        column_name: str
        data_type: polars.datatypes.DataType

        column_metrics = []
        for column_name, data_type in dataframe.schema.items():
            if column_name in columns_to_ignore:
                continue

            handler: Callable[[polars.DataFrame, str], BaseProfile] = self._get_handler(
                data_type
            )

            column_metrics.append(
                ColumnData(
                    name=column_name,
                    type=str(data_type),
                    metrics=handler(dataframe, column_name),
                )
            )

        return column_metrics

    def run_profiling(
        self,
        dataframe: polars.DataFrame,
        columns_to_ignore: list[str] | None = None,
    ) -> ProfilerOutput:
        _columns_to_ignore: list[str] = columns_to_ignore if columns_to_ignore else []

        self._validate_row_count_not_zero(dataframe)
        self._validate_columns_to_profile(dataframe, _columns_to_ignore)

        columns_data = self._gather_columns_metrics(dataframe, _columns_to_ignore)
        rows_count, column_count = dataframe.shape
        results = ProfilerOutput(
            rows_count=rows_count,
            column_count=column_count,
            ignored_columns=_columns_to_ignore,
            dataframe_size=dataframe.estimated_size(),
            columns=columns_data,
        )
        return results



import dataclasses

from plmdp.models.type_agnostic import BaseProfile
from plmdp.primitives import OptionalNumeric


@dataclasses.dataclass
class NumericProfile(BaseProfile):
    mean: OptionalNumeric
    median: OptionalNumeric
    std: OptionalNumeric
    min: OptionalNumeric
    max: OptionalNumeric

    percentile25: OptionalNumeric
    percentile50: OptionalNumeric
    percentile75: OptionalNumeric



import dataclasses
from datetime import datetime, timezone

from plmdp.models import BaseProfile


@dataclasses.dataclass
class ColumnData:
    name: str
    type: str
    metrics: BaseProfile


@dataclasses.dataclass
class ProfilerOutput:
    rows_count: int
    column_count: int
    ignored_columns: list[str]
    dataframe_size: int | float
    columns: list[ColumnData]
    created_at: datetime = dataclasses.field(
        default_factory=lambda: datetime.now(timezone.utc), init=False
    )



import datetime

from freezegun import freeze_time
from polars import String, Date, Float32, read_csv
import pytest

from plmdp.models import DateOrDateTimeProfile, NumericProfile
from plmdp.models.output import ProfilerOutput, ColumnData
from plmdp.profiler import Profiler
from tests.constants import TESTS_RESOURCES


@pytest.fixture
@freeze_time("2000-01-01")
def profiler_output() -> ProfilerOutput:
    return ProfilerOutput(
        rows_count=20,
        column_count=3,
        ignored_columns=["comment"],
        dataframe_size=545,
        columns=[
            ColumnData(
                name="dob",
                type="Date",
                metrics=DateOrDateTimeProfile(
                    nulls_count=0,
                    min=datetime.date(1973, 9, 19),
                    max=datetime.date(1999, 8, 29),
                ),
            ),
            ColumnData(
                name="sales",
                type="Float32",
                metrics=NumericProfile(
                    nulls_count=0,
                    mean=181.5,
                    median=185.0,
                    std=78.42,
                    min=50.0,
                    max=320.0,
                    percentile25=120.0,
                    percentile50=190.0,
                    percentile75=230.0,
                ),
            ),
        ],
    )


@freeze_time("2000-01-01")
def test_profiler(profiler_output: ProfilerOutput) -> None:
    df = read_csv(
        (TESTS_RESOURCES / "data.csv"),
        schema={"comment": String, "dob": Date, "sales": Float32},
    )

    result = Profiler().run_profiling(
        df,
        columns_to_ignore=[
            "comment",
        ],
    )

    assert result == profiler_output



import dataclasses


from plmdp.models.type_agnostic import BaseProfile
from plmdp.primitives import OptionalInt, OptionalNumeric


@dataclasses.dataclass
class StringProfile(BaseProfile):
    min_length: OptionalInt
    max_length: OptionalInt
    avg_length: OptionalNumeric
    median_length: OptionalNumeric

    min_token_count: OptionalNumeric
    max_token_count: OptionalNumeric
    avg_token_count: OptionalNumeric
    median_token_count: OptionalNumeric

    empty_or_whitespace_count: OptionalInt



class UnsupportedDataTypeException(Exception):
    """Handles unsupported column data type."""


class EmptyDataFrameException(Exception):
    """Handles empty dataframe passed to profiling."""


class NoColumnsToProfile(Exception):
    """Handles situations when all dataframe's columns are ignored."""


class UnsupportedDataFileExtension(Exception):
    """Handles file extensions that differ from csv and parquet."""



from functools import lru_cache
from typing import Callable

import polars
from polars import DataType

from plmdp.enums import SupportedDataTypes
from plmdp.exceptions import UnsupportedDataTypeException

from plmdp.models.type_agnostic import BaseProfile
from polars.datatypes import (
    Decimal,
    Float32,
    Float64,
    Int8,
    Int16,
    Int32,
    Int64,
    Int128,
    UInt8,
    UInt16,
    UInt32,
    UInt64,
    String,
    Categorical,
    Date,
    Datetime,
)

from plmdp.service import (
    get_numeric_metrics,
    get_string_metrics,
    get_type_agnostic_metrics,
    get_date_or_datetime_metrics,
)


class MetricsHandlerFactory:
    TYPE_MAPPING: dict[SupportedDataTypes, list[type[DataType]]] = {
        SupportedDataTypes.NUMERIC: [
            Decimal,
            Float32,
            Float64,
            Int8,
            Int16,
            Int32,
            Int64,
            Int128,
            UInt8,
            UInt16,
            UInt32,
            UInt64,
        ],
        SupportedDataTypes.STRING: [String, Categorical],
        SupportedDataTypes.DATE_OR_DATETIME: [Date, Datetime],
    }
    HANDLER_MAPPING: dict[
        SupportedDataTypes, Callable[[polars.DataFrame, str], BaseProfile]
    ] = {
        SupportedDataTypes.NUMERIC: get_numeric_metrics,
        SupportedDataTypes.STRING: get_string_metrics,
        SupportedDataTypes.DATE_OR_DATETIME: get_date_or_datetime_metrics,
        SupportedDataTypes.BASE: get_type_agnostic_metrics,
    }
    FALLBACK_TYPE = SupportedDataTypes.BASE

    @lru_cache
    def get_profile_type(self, _type: DataType) -> SupportedDataTypes:
        for simplified_type, polar_types in self.TYPE_MAPPING.items():
            if _type in polar_types:
                return simplified_type
        if isinstance(_type, DataType):
            return self.FALLBACK_TYPE
        raise UnsupportedDataTypeException(f"Unsupported datatype {_type}")

    def create(
        self, data_type: DataType
    ) -> Callable[[polars.DataFrame, str], BaseProfile]:
        profile_type: SupportedDataTypes = self.get_profile_type(data_type)
        return self.HANDLER_MAPPING[profile_type]



import argparse
import json
from pathlib import Path
from typing import Any, cast

import polars.datatypes

from plmdp.cli.formatter import FormatterFactory
from plmdp.exceptions import UnsupportedDataTypeException
from plmdp.cli.data_loader import str_type_to_polars, DataReaderFactory
from plmdp.models.output import ProfilerOutput
from plmdp.primitives import ColumnName
from plmdp.profiler import Profiler


def load_schema_from_cli(
    schema: str | None,
) -> dict[ColumnName, polars.datatypes.DataTypeClass] | None:
    if not schema:
        return None
    try:
        json_input = json.loads(schema)
        return {k: str_type_to_polars(v) for k, v in json_input.items()}

    except json.JSONDecodeError as e:
        raise ValueError("Schema is not valid json") from e

    except UnsupportedDataTypeException:
        raise


def load_kwargs(kwargs_str: str | None) -> dict[Any, Any]:
    if not kwargs_str:
        return {}
    try:
        return cast(dict[Any, Any], json.loads(kwargs_str))
    except json.JSONDecodeError as e:
        raise ValueError("Kwargs are invalid json") from e


def main() -> str:
    parser = argparse.ArgumentParser(description="Profiler cli")

    parser.add_argument("-p", "--path", required=True, help="Path to datafile")

    parser.add_argument(
        "-s", "--schema", required=False, help="Dataframe schema as json"
    )

    parser.add_argument(
        "-k", "--kwargs", required=False, help="Polars dataframe loader kwargs"
    )

    parser.add_argument(
        "-c", "--columns-to-ignore", required=False, help="Column names to ignore"
    )

    parser.add_argument(
        "-f",
        "--formatter",
        required=False,
        default="json",
        help="Stdout format: json, yaml or raw",
    )
    args = parser.parse_args()
    str_schema = args.schema
    path = Path(args.path)
    dataloader_kwargs = args.kwargs
    columns_to_ignore = (
        args.columns_to_ignore.split(",") if args.columns_to_ignore else []
    )

    schema = load_schema_from_cli(str_schema)
    kwargs = load_kwargs(dataloader_kwargs)
    stdout_format = args.formatter

    read_data = DataReaderFactory.create(path)
    data = read_data(source=path, schema=schema, **kwargs)
    results: ProfilerOutput = Profiler().run_profiling(
        data, columns_to_ignore=columns_to_ignore
    )
    formatter = FormatterFactory.get_formatter(stdout_format)
    return formatter(results)


if __name__ == "__main__":
    print(main())



from typing import cast

import polars
from plmdp.models._string import StringProfile
from plmdp.primitives import OptionalInt, OptionalNumeric


def get_string_metrics(
    dataframe: polars.DataFrame,
    column_name: str,
) -> StringProfile:
    column = dataframe[column_name]
    if column.dtype == polars.Categorical:
        column = column.cast(polars.String)

    null_count = column.null_count()

    empty_or_whitespace_count: int = cast(int, (column.str.strip_chars() == "").sum())

    string_lengths = column.str.len_chars()

    token_counts = column.str.split(by=" ").list.len()

    min_length = cast(OptionalInt, string_lengths.min())
    max_length = cast(OptionalInt, string_lengths.max())
    avg_length = cast(
        OptionalNumeric,
        string_lengths.mean() if not string_lengths.is_empty() else None,
    )
    median_length = cast(
        OptionalNumeric,
        string_lengths.median() if not string_lengths.is_empty() else None,
    )

    min_token_count = cast(OptionalNumeric, token_counts.min())
    max_token_count = cast(OptionalNumeric, token_counts.max())
    avg_token_count = cast(
        OptionalNumeric, token_counts.mean() if not token_counts.is_empty() else None
    )
    median_token_count = cast(
        OptionalNumeric, token_counts.median() if not token_counts.is_empty() else None
    )

    return StringProfile(
        nulls_count=null_count,
        min_length=min_length,
        max_length=max_length,
        avg_length=avg_length,
        median_length=median_length,
        min_token_count=min_token_count,
        max_token_count=max_token_count,
        avg_token_count=avg_token_count,
        median_token_count=median_token_count,
        empty_or_whitespace_count=empty_or_whitespace_count,
    )



from typing import cast

import polars

from plmdp.constants import FLOATING_POINT
from plmdp.models.numeric import NumericProfile
from plmdp.primitives import OptionalNumeric


def get_numeric_metrics(
    dataframe: polars.DataFrame,
    column_name: str,
) -> NumericProfile:
    column = dataframe[column_name]
    null_count = column.null_count()
    mean = cast(OptionalNumeric, column.mean())
    median = cast(OptionalNumeric, column.median())
    std = cast(OptionalNumeric, column.std())
    min_value = cast(OptionalNumeric, column.min())
    max_value = cast(OptionalNumeric, column.max())

    percentile25 = cast(OptionalNumeric, column.quantile(0.25))
    percentile50 = cast(OptionalNumeric, column.quantile(0.50))
    percentile75 = cast(OptionalNumeric, column.quantile(0.75))

    return NumericProfile(
        nulls_count=null_count,
        mean=round(mean, FLOATING_POINT) if mean is not None else None,
        median=round(median, FLOATING_POINT) if median is not None else None,
        std=round(std, FLOATING_POINT) if std is not None else None,
        min=round(min_value, FLOATING_POINT) if min_value is not None else None,
        max=round(max_value, FLOATING_POINT) if max_value is not None else None,
        percentile25=round(percentile25, FLOATING_POINT)
        if percentile25 is not None
        else None,
        percentile50=round(percentile50, FLOATING_POINT)
        if percentile50 is not None
        else None,
        percentile75=round(percentile75, FLOATING_POINT)
        if percentile75 is not None
        else None,
    )



from plmdp.models.type_agnostic import BaseProfile
from plmdp.models.date_or_datetime import DateOrDateTimeProfile
from plmdp.models._string import StringProfile
from plmdp.models.numeric import NumericProfile

__all__ = ["NumericProfile", "StringProfile", "DateOrDateTimeProfile", "BaseProfile"]



import polars
import pytest
from _pytest.logging import LogCaptureFixture

from plmdp.exceptions import EmptyDataFrameException, NoColumnsToProfile
from plmdp.profiler import Profiler


@pytest.fixture
def profiler() -> Profiler:
    return Profiler()


@pytest.fixture
def sample_dataframe() -> polars.DataFrame:
    return polars.DataFrame({"a": [1, 2], "b": [3, 4], "c": [5, 6]})


def test_validate_row_count_not_zero_passes_on_non_empty_dataframe(
    profiler: Profiler,
) -> None:
    non_empty_df = polars.DataFrame({"column": [1, 2, 3]})
    profiler._validate_row_count_not_zero(non_empty_df)


def test_validate_row_count_not_zero_raises_exception_on_empty_dataframe(
    caplog: LogCaptureFixture,
    profiler: Profiler,
) -> None:
    empty_df = polars.DataFrame()
    with pytest.raises(EmptyDataFrameException, match="Passed dataframe is empty."):
        profiler._validate_row_count_not_zero(empty_df)

    assert "Passed dataframe is empty." in caplog.text


def test_all_columns_ignored(
    sample_dataframe: polars.DataFrame,
    profiler: Profiler,
) -> None:
    columns_to_ignore = ["a", "b", "c"]

    with pytest.raises(NoColumnsToProfile, match="All columns are ignored"):
        profiler._validate_columns_to_profile(sample_dataframe, columns_to_ignore)


def test_some_columns_ignored(
    sample_dataframe: polars.DataFrame,
    profiler: Profiler,
) -> None:
    columns_to_ignore = ["a"]

    profiler._validate_columns_to_profile(sample_dataframe, columns_to_ignore)


def test_no_columns_ignored(
    sample_dataframe: polars.DataFrame,
    profiler: Profiler,
) -> None:
    columns_to_ignore: list[str] = []

    profiler._validate_columns_to_profile(sample_dataframe, columns_to_ignore)


def test_extra_ignored_columns(
    sample_dataframe: polars.DataFrame,
    caplog: LogCaptureFixture,
    profiler: Profiler,
) -> None:
    columns_to_ignore = ["a", "d"]

    profiler._validate_columns_to_profile(sample_dataframe, columns_to_ignore)

    assert (
        "Warning: The following columns in 'columns_to_ignore' are not in the DataFrame: {'d'}"
        in caplog.text
    )



from typing import Callable

import pytest
from polars import DataType

from plmdp.enums import SupportedDataTypes
from plmdp.exceptions import UnsupportedDataTypeException
from plmdp.models import BaseProfile
from plmdp.profile_factory import MetricsHandlerFactory
from plmdp.service import (
    get_numeric_metrics,
    get_string_metrics,
    get_type_agnostic_metrics,
    get_date_or_datetime_metrics,
)
import polars
from polars.datatypes import (
    Decimal,
    Float32,
    Int8,
    String,
    Date,
    Datetime,
    Boolean,
    DataTypeClass,
)


@pytest.fixture
def metrics_handler() -> MetricsHandlerFactory:
    return MetricsHandlerFactory()


@pytest.mark.parametrize(
    "data_type, expected_profile_type",
    [
        (Float32, SupportedDataTypes.NUMERIC),
        (Int8, SupportedDataTypes.NUMERIC),
        (Decimal, SupportedDataTypes.NUMERIC),
        (String, SupportedDataTypes.STRING),
        (Date, SupportedDataTypes.DATE_OR_DATETIME),
        (Datetime, SupportedDataTypes.DATE_OR_DATETIME),
    ],
)
def test_get_profile_type(
    metrics_handler: MetricsHandlerFactory,
    data_type: DataTypeClass,
    expected_profile_type: SupportedDataTypes,
) -> None:
    assert metrics_handler.get_profile_type(data_type) == expected_profile_type


def test_get_profile_type_unsupported(metrics_handler: MetricsHandlerFactory) -> None:
    with pytest.raises(UnsupportedDataTypeException):
        metrics_handler.get_profile_type(str())


@pytest.mark.parametrize(
    "data_type, expected_handler",
    [
        (Float32, get_numeric_metrics),
        (Int8, get_numeric_metrics),
        (String, get_string_metrics),
        (Date, get_date_or_datetime_metrics),
        (Datetime, get_date_or_datetime_metrics),
    ],
)
def test_create(
    metrics_handler: MetricsHandlerFactory,
    data_type: DataType,
    expected_handler: Callable[[polars.DataFrame, str], BaseProfile],
) -> None:
    handler = metrics_handler.create(data_type)
    assert handler == expected_handler


def test_create_fallback(metrics_handler: MetricsHandlerFactory) -> None:
    handler = metrics_handler.create(Boolean())
    assert handler == get_type_agnostic_metrics



from pathlib import Path
from typing import Callable

import polars.datatypes
from polars.datatypes import (
    Decimal,
    Float32,
    Float64,
    Int8,
    Int16,
    Int32,
    Int64,
    Int128,
    UInt8,
    UInt16,
    UInt32,
    UInt64,
    String,
    Categorical,
    Date,
    Datetime,
)

from plmdp.exceptions import UnsupportedDataTypeException, UnsupportedDataFileExtension

STR_TO_POLARS_TYPE = {
    "Decimal": Decimal,
    "Float32": Float32,
    "Float64": Float64,
    "Int8": Int8,
    "Int16": Int16,
    "Int32": Int32,
    "Int64": Int64,
    "Int128": Int128,
    "UInt8": UInt8,
    "UInt16": UInt16,
    "UInt32": UInt32,
    "UInt64": UInt64,
    "String": String,
    "Categorical": Categorical,
    "Date": Date,
    "Datetime": Datetime,
}


def str_type_to_polars(str_type: str) -> polars.datatypes.DataTypeClass:
    try:
        return STR_TO_POLARS_TYPE[str_type]
    except KeyError as e:
        raise UnsupportedDataTypeException(f"Unsupported datatype {str_type}") from e


class DataReaderFactory:
    @staticmethod
    def create(path: Path) -> Callable[..., polars.DataFrame]:
        file_extension = path.suffix

        readers: dict[str, Callable[..., polars.DataFrame]] = {
            ".csv": polars.read_csv,
            ".parquet": polars.read_parquet,
        }

        try:
            reader = readers[file_extension]
        except KeyError as e:
            raise UnsupportedDataFileExtension(
                "Supported file extensions are: '.csv' and '.parquet'"
            ) from e

        return reader



import polars

from plmdp.models.type_agnostic import BaseProfile


def get_type_agnostic_metrics(
    dataframe: polars.DataFrame,
    column_name: str,
) -> BaseProfile:
    column = dataframe[column_name]
    null_count = column.null_count()

    return BaseProfile(
        nulls_count=null_count,
    )



from pathlib import Path

import pytest
import polars as pl

from plmdp.exceptions import UnsupportedDataFileExtension, UnsupportedDataTypeException
from plmdp.cli.data_loader import DataReaderFactory, str_type_to_polars


def test_str_type_to_polars_valid() -> None:
    assert str_type_to_polars("Float32") == pl.Float32
    assert str_type_to_polars("Int64") == pl.Int64
    assert str_type_to_polars("String") == pl.String


def test_str_type_to_polars_invalid() -> None:
    with pytest.raises(
        UnsupportedDataTypeException, match="Unsupported datatype UnknownType"
    ):
        str_type_to_polars("UnknownType")


def test_data_reader_factory_csv() -> None:
    reader = DataReaderFactory.create(Path("data.csv"))
    assert reader == pl.read_csv


def test_data_reader_factory_parquet() -> None:
    reader = DataReaderFactory.create(Path("data.parquet"))
    assert reader == pl.read_parquet


def test_data_reader_factory_unsupported() -> None:
    with pytest.raises(
        UnsupportedDataFileExtension,
        match="Supported file extensions are: '.csv' and '.parquet'",
    ):
        DataReaderFactory.create(Path("data.txt"))



from typing import cast

import polars

from plmdp.models.date_or_datetime import DateOrDateTimeProfile
from plmdp.primitives import OptionalDateOrDateTime


def get_date_or_datetime_metrics(
    dataframe: polars.DataFrame,
    column_name: str,
) -> DateOrDateTimeProfile:
    column = dataframe[column_name]
    null_count = column.null_count()
    min_value = cast(OptionalDateOrDateTime, column.min())
    max_value = cast(OptionalDateOrDateTime, column.max())

    return DateOrDateTimeProfile(nulls_count=null_count, min=min_value, max=max_value)



import argparse
from pathlib import Path

import pytest
import json
from unittest.mock import patch, MagicMock

from polars import Int64, String

from plmdp.cli.cli import load_schema_from_cli, load_kwargs, main
from plmdp.exceptions import UnsupportedDataTypeException
from plmdp.models.output import ProfilerOutput
from tests.test_profiler import profiler_output  # noqa: F401


def test_load_schema_from_cli_valid() -> None:
    schema_json = json.dumps({"column1": "Int64", "column2": "String"})
    result = load_schema_from_cli(schema_json)
    assert result == {"column1": Int64, "column2": String}


def test_load_schema_from_cli_invalid_json() -> None:
    with pytest.raises(ValueError, match="Schema is not valid json"):
        load_schema_from_cli("{]")


def test_load_schema_from_cli_unsupported_data_type() -> None:
    with pytest.raises(UnsupportedDataTypeException):
        load_schema_from_cli(json.dumps({"column1": "InvalidType"}))


def test_load_schema_from_cli_none() -> None:
    assert load_schema_from_cli(None) is None


def test_load_kwargs_valid() -> None:
    kwargs_json = json.dumps({"sep": ",", "header": True})
    result = load_kwargs(kwargs_json)
    assert result == {"sep": ",", "header": True}


def test_load_kwargs_invalid_json() -> None:
    with pytest.raises(ValueError, match="Kwargs are invalid json"):
        load_kwargs("invalid_json")


@patch("argparse.ArgumentParser.parse_args")
@patch("plmdp.cli.data_loader.DataReaderFactory.create")
@patch("plmdp.profiler.Profiler.run_profiling")
@patch("plmdp.cli.formatter.FormatterFactory.get_formatter")
def test_main_with_formatters(
        mock_get_formatter: MagicMock,
        mock_run_profiling: MagicMock,
        mock_data_reader: MagicMock,
        mock_parse_args: MagicMock,
        profiler_output: ProfilerOutput,  # noqa: F811
) -> None:
    mock_parse_args.return_value = argparse.Namespace(
        path="/fake/path.csv", schema=None, kwargs=None, columns_to_ignore=None, formatter="json"
    )

    mock_data_reader.return_value = lambda source, schema, **kwargs: "mock_dataframe"
    mock_run_profiling.return_value = profiler_output

    mock_formatter = MagicMock()
    mock_get_formatter.return_value = mock_formatter

    mock_parse_args.return_value.formatter = "json"
    main()
    mock_get_formatter.assert_called_with("json")
    mock_formatter.assert_called_with(profiler_output)

    mock_data_reader.assert_called_with(Path("/fake/path.csv"))
    mock_run_profiling.assert_called_with("mock_dataframe", columns_to_ignore=[])



from dataclasses import asdict
from typing import Literal, Callable
import json
import yaml

from plmdp.models.output import ProfilerOutput


class FormatterFactory:
    FORMATTERS = {
        "raw": lambda data: repr(data),
        "json": lambda data: json.dumps(asdict(data), default=str, indent=2),
        "yaml": lambda data: yaml.dump(asdict(data)),
    }

    @staticmethod
    def get_formatter(
        print_format: Literal["raw", "yaml", "json"] = "raw",
    ) -> Callable[[ProfilerOutput], str]:
        try:
            return FormatterFactory.FORMATTERS[print_format]
        except KeyError:
            raise ValueError("Unsupported format. Use 'raw', 'json', or 'yaml'.")



from plmdp.service.numeric import get_numeric_metrics
from plmdp.service.date_or_datetime import get_date_or_datetime_metrics
from plmdp.service._string import get_string_metrics
from plmdp.service.type_agnostic import get_type_agnostic_metrics

__all__ = [
    "get_string_metrics",
    "get_numeric_metrics",
    "get_date_or_datetime_metrics",
    "get_type_agnostic_metrics",
]
